#define EXT_BUFSIZE     16
extern uint8_t ext_buf[];

#define DEV_SYS		0
#define DEV_ADC		1
#define DEV_OSC		2
#define DEV_FCNT	3
#define DEV_ECHO	9
#define DEV_I2C		10

/* i2c functions */
extern void init_i2c();
extern int8_t i2c_send(uint8_t dev, uint8_t size);
extern int8_t i2c_recv(uint8_t dev, uint8_t size);

/* ADC functions */

#define ADC_CHAN_AIN1		0x0 // external #1
#define ADC_CHAN_AIN2		0x1 // external #2
#define ADC_CHAN_AIN3		0x2 // external #3
#define ADC_CHAN_AIN4		0x3 // external #4
#define ADC_CHAN_AIN5		0x4 // internal #1 (CURRENT +)
#define ADC_CHAN_AIN6		0x5 // internal #2 (CURRENT -)
#define ADC_CHAN_VCC		0x6 // internal #3 (VCC x k)
#define ADC_CHAN_TEMP		0x7 // internal #4 (TEMP)

#define ADC_CHAN_AIN56		0x8 //internal
#define ADC_CHAN_CURRENT	0x8 //internal
#define ADC_CHAN_AIN12		0x9
#define ADC_CHAN_AIN23		0xa
#define ADC_CHAN_AIN31		0xb

#define ADC_CHAN_AIN65		0xc //internal reverse input
#define ADC_CHAN_AIN21		0xd // reverse input
#define ADC_CHAN_AIN32		0xe // reverse input
#define ADC_CHAN_AIN13		0xf // reverse input

#define ADC_CHAN_DELTA		0x08
#define ADC_CHAN_MASK		0x0f

/* flags for DELTA mode */
#define ADC_RANGE_AUTO		0x00 // Auto Range
#define ADC_RANGE_V512		0x10 // Reference VCC (+ calibrate VCC)
#define ADC_RANGE_V256		0x20 // Reference 2.56V
#define ADC_RANGE_V110		0x30 // Reference 1.10V
#define ADC_RANGE_V032		0x50 // Reference 2.56V + Gain x8
#define ADC_RANGE_V008		0x70 // Reference 2.56V + Gain x32
#define ADC_RANGE_MASK		0x70
#define ADC_BIPOLAR		0x80 // use BIN

/* result format 
 *
 *  int16_t: unit 0.15625 mV (10/64 mV)
 *
 *   +-+-+----------+---+
 *   |1|1|  10      | 4 |
 *   +-+-+----------+---+
 *       |          |
 *
 */

extern void init_adc();
extern int8_t adc_send(uint8_t size);
extern int8_t adc_recv(uint8_t size);

/* fcnt functions */

extern int8_t fcnt_send(uint8_t size);
extern int8_t fcnt_recv(uint8_t size);

#define FCNT_START	0x01

/* osc functions */

extern void init_osc();
extern void term_osc();
extern int8_t osc_send(uint8_t size);
extern int8_t osc_recv(uint8_t size);

#define OSC_INIT	0x00
#define OSC_START	0x01
#define OSC_STOP	0x02
#define OSC_SET	0x03

